#include <stdlib.h>
#include "block.h"
#include "game.h"


pointlist grayblockinfo[4] =
  {
  {{ {2,0}, {1,0}, {0,0},   {-1,0}, },},     //            X          X
  {{ {0,2}, {0,1}, {0, 0},  {0,-1}, },},     //   XXXX     X   XXXX   X
  {{ {1,0}, {0,0}, {-1, 0}, {-2,0}, },},     //            X          X
  {{ {0,1}, {0,0}, { 0,-1}, {0,-2}, },},     //            X          X
  };

pointlist smokblockinfo[4] =
  {
  {{ {0,0}, {1,0}, {0,1}, {1,1}, },},        //    XX   XX   XX   XX
  {{ {0,0}, {1,0}, {0,1}, {1,1}, },},        //    XX   XX   XX   XX
  {{ {0,0}, {1,0}, {0,1}, {1,1}, },},        //
  {{ {0,0}, {1,0}, {0,1}, {1,1}, },},        //
  };

pointlist blueblockinfo[4] =
  {
  {{ {-1,0},  {0,0}, {1,0}, {-1,1}, },},     //   XXX   XX    X  X
  {{ {-1,-1}, {0,-1},{0,0}, {0,1},  },},     //   X      X  XXX  X
  {{ {-1,0},  {0,0}, {1,0}, {1,-1}, },},     //          X       XX
  {{ {0,-1},  {0,0}, {0,1}, {1,1},  },},
  };

pointlist yellblockinfo[4] =
  {
  {{ {-1,0},  {0,0},  {1,0}, {1,1}, },},     //   XXX    X   X     XX
  {{ {0,-1},  {0,0},  {0,1}, {-1,1}, },},    //     X    X   XXX   X
  {{ {-1,-1}, {-1,0}, {0,0}, {1,0}, },},     //         XX         X
  {{ {0,-1},  {1,-1}, {0,0}, {0,1}, },},
  };

pointlist jadeblockinfo[4] =
  {
  {{ {-1,0}, {0,0},  {1,0}, {0,1}, },},     //   XXX    X    X    X
  {{ {0,-1}, {0,0},  {0,1}, {-1,0}, },},    //    X    XX   XXX   XX
  {{ {0,-1}, {-1,0}, {0,0},  {1,0}, },},    //          X         X
  {{ {0,-1}, {0,0}, {0,1}, {1,0}, },},
  };

pointlist pinkblockinfo[4] =
  {
  {{ { 0,0},  {1,0},  {0,1}, {-1,1}, },},    //     XX   X     XX  X
  {{ {-1,-1}, {-1,0}, {0,0}, {0,1}, },},     //    XX    XX   XX   XX
  {{ {-1,-0}, {0,-1}, {0,0}, {1,-1}, },},    //           X         X
  {{ {-1,-1}, {-1,0}, {0,0}, {0,1}, },},
  };

pointlist turqblockinfo[4] =
  {
  {{ { -1,0},  {0,0},  {0,1},  {1,1}, },},    //    XX     X   XX     X
  {{ { 0,-1},  { 0,0}, {-1,0}, {-1,1}, },},   //     XX   XX    XX   XX
  {{ { -1,0},  {0,0},  {0,1},  {1,1}, },},    //          X          X
  {{ { 0,-1},  { 0,0}, {-1,0}, {-1,1}, },},
  };

Block::Block(int xx,int yy,int r,int i,pointlist* p)
  {
  initx=xx;
  inity=yy;
  initrot=r;
  image=i;
  listptr=p;
  }

void Block::initialize()
  {
  load_gfxlib("blocks.gfx");
  load_sfxlib("sounds.sfx");
  numclips=get_num_clips();
  }

void Block::play_clip(int clip)
  {
  if (clip==ROWREMOVAL)
    play_sound_clip(clip);
  else
    {
    int r=rand()%(numclips-1);
    play_sound_clip(clip+r);
    }
  }

void Block::drop()
  {
  unset();
  for (int i=1;i<PIT_DEPTH;i++)
    {
    if (!can_set(x,y+i))
      break;
    }
  ox=x;
  oy=y;
  oldrot=rot;
  y+=i-1;
  set();
  }

void Block::draw(int x,int y)
  {
  show_image(x,y,image);
  }

void Block::reset()
  {
  x=initx;
  y=inity;
  ox=x;
  oy=y;
  rot=initrot;
  oldrot=rot;
  if (can_set(x,y+1)==FALSE)
    post_message(GAME_OVER);
  set();
  }

void Block::move(int dir)
  {
  switch (dir)
    {
    case LEFT:
      unset();
      ox=x;
      oy=y;
      x-=1;
      oldrot=rot;
      set();
      break;
    case RIGHT:
      unset();
      ox=x;
      oy=y;
      x+=1;
      oldrot=rot;
      set();
      break;
    case DOWN:
      unset();
      ox=x;
      oy=y;
      y+=1;
      oldrot=rot;
      set();
      break;
    }
  }

void Block::rotate()
  {
  unset();
  oldrot=rot;
  rot=(rot+1)%4;
  ox=x;
  oy=y;
  set();
  }

int Block::can_move(int dir)
  {
  int ret=FALSE;
  switch (dir)
    {
    case LEFT:
      unset();
      if (can_set(x-1,y))  ret=TRUE;
      set();
      break;
    case RIGHT:
      unset();
      if (can_set(x+1,y))  ret=TRUE;
      set();
      break;
    case DOWN:
      unset();
      if (can_set(x,y+1))  ret=TRUE;
      set();
      break;
    }
  return ret;
  }

int Block::can_rotate()
  {
  int ret=FALSE;
  int r=(rot+1)%4;
  unset();
  if (can_set(r))  ret=TRUE;
  set();
  return ret;
  }

JadeBlock::JadeBlock() : Block(4, 0,0,1,jadeblockinfo)  { }
BlueBlock::BlueBlock() : Block(4, 0,0,2,blueblockinfo)  { }
GrayBlock::GrayBlock() : Block(4, 0,0,3,grayblockinfo)  { }
PinkBlock::PinkBlock() : Block(4, 0,0,4,pinkblockinfo)  { }
YellBlock::YellBlock() : Block(4, 0,0,5,yellblockinfo)  { }
SmokBlock::SmokBlock() : Block(4, 0,0,6,smokblockinfo)  { }
TurqBlock::TurqBlock() : Block(4, 0,0,7,turqblockinfo)  { }
