#include <theatrix.h>

class SoundTech : public VocalHand	{
	char *sfxfile;
	void initialize()
		{ load_sfxlib(sfxfile); }
public:
	SoundTech(char *sfx)
		{ sfxfile = sfx; }
};

class Town : public SceneryDirector	{
	static int townct;
	int tune;
	int clip;
	virtual void look_forward()	{ }
	virtual void look_right()  	{ }
	virtual void look_left()   	{ }
	virtual void look_back()   	{ }
	void on_escape();
	DECLARE_CUELIST
	DECLARE_MOUSECURSORS
protected:
	static MusicHand *conductor;
	static SoundTech *soundtech;
	Town(char *pcx, int tn = 0, int cl = 0);
	~Town();
	virtual void display();
	virtual void hide();
	void on_timer();
};

MusicHand *Town::conductor;
SoundTech *Town::soundtech;
int Town::townct;

Town::Town(char *pcx, int tn, int cl) :
				SceneryDirector(pcx, NoTransition)
{
	if (townct == 0)	{
		conductor = new MusicHand("town.xmi");
		soundtech = new SoundTech("town.sfx");
	}
	tune = tn;
	clip = cl;
	townct++;
}
Town::~Town()
{
	if (--townct == 0)	{
		delete soundtech;
		delete conductor;
	}
}

CUELIST(Town)
	KEYSTROKE(LF,  look_left)
	KEYSTROKE(UP,  look_forward)
	KEYSTROKE(DN,  look_back)
	KEYSTROKE(RT,  look_right)
	KEYSTROKE(ESC, on_escape)
    TIMER(18, on_timer)
ENDCUELIST

CURSORLIST(Town)
	MOUSE_CURSOR(  0,   0, 105, 239, LFCURSOR, look_left)
	MOUSE_CURSOR(106,   0, 211, 199, UPCURSOR, look_forward)
	MOUSE_CURSOR(106, 200, 211, 239, DNCURSOR, look_back)
	MOUSE_CURSOR(212,   0, 319, 239, RTCURSOR, look_right)
ENDCURSORLIST

void Town::display()
{
	SceneryDirector::display();
	mouse_visible();
	if (tune)
		conductor->play_music_clip(tune);
	if (clip)
		soundtech->play_sound_clip(clip);
}
void Town::hide()
{
	if (tune)
		conductor->stop_music_clip();
	if (clip)
		soundtech->stop_sound_clip();
	mouse_invisible();
	SceneryDirector::hide();
}
void Town::on_timer()
{
	if (clip && !soundtech->sound_clip_is_playing())
		soundtech->play_sound_clip(clip);
}
void Town::on_escape()
{
	set_next_director(&typeid(StopDirector));
	stop_director();
}

class Town1;
class Town2;
class Town3;
class Town4;
class Town5;
class Town6;
class Town7;
class Town8;
class Town9;
class Town10;
class Town11;
class Town12;

class Town1 : public Town	{
public:
	Town1() : Town("town01.pcx") { }
	void look_forward() { start_director(typeid(Town2)); }
};
class Town2 : public Town	{
public:
	Town2() : Town("town02.pcx") { }
	void look_forward() { start_director(typeid(Town4)); }
	void look_right()   { start_director(typeid(Town3)); }
	void look_back()    { start_director(typeid(Town1)); }
};
class Town3 : public Town	{
public:
	Town3() : Town("town03.pcx", 4, 2) { }
	void look_left()  { start_director(typeid(Town2)); }
	void look_right() { start_director(typeid(Town10)); }
};
class Town4 : public Town	{
public:
	Town4() : Town("town04.pcx") { }
	void look_forward() { start_director(typeid(Town7)); }
	void look_left()    { start_director(typeid(Town6)); }
	void look_right()   { start_director(typeid(Town5)); }
	void look_back()    { start_director(typeid(Town2)); }
};
class Town5 : public Town	{
public:
	Town5() : Town("town05.pcx") { }
	void look_forward() { start_director(typeid(Town11)); }
	void look_right() { start_director(typeid(Town9)); }
	void look_left()  { start_director(typeid(Town4)); }
};
class Town6 : public Town	{
public:
	Town6() : Town("town06.pcx", 3) { }
	void look_right() { start_director(typeid(Town4)); }
	void look_left()  { start_director(typeid(Town9)); }
};
class Town7 : public Town	{
public:
	Town7() : Town("town07.pcx", 1) { }
	void look_right() { start_director(typeid(Town8)); }
	void look_left()  { start_director(typeid(Town8)); }
	void look_back()  { start_director(typeid(Town4)); }
};
class Town8 : public Town	{
public:
	Town8() : Town("town08.pcx") { }
	void look_forward() { start_director(typeid(Town9)); }
};
class Town9 : public Town	{
public:
	Town9() : Town("town09.pcx") { }
	void look_forward() { start_director(typeid(Town10)); }
	void look_right()   { start_director(typeid(Town6)); }
	void look_left()    { start_director(typeid(Town5)); }
	void look_back()    { start_director(typeid(Town8)); }
};
class Town10 : public Town	{
public:
	Town10() : Town("town10.pcx", 0, 1) { }
	void look_right() { start_director(typeid(Town1)); }
	void look_left()  { start_director(typeid(Town3)); }
	void look_back()  { start_director(typeid(Town9)); }
};
class Town11 : public Town	{
public:
	Town11() : Town("town11.pcx") { }
	void look_back()  { start_director(typeid(Town5)); }
	void look_forward() { start_director(typeid(Town12)); }
};
class Town12 : public Town	{
public:
	Town12() : Town("town11.pcx", 2) { }
	void display()
	{
		Town::display();
		show_video("room.flc", 132, 53, TRUE);
	}
	void hide()
	{
		stop_video();
		Town::hide();
	}
	void look_back()  { start_director(typeid(Town11)); }
	void iterate_director()
	{
		SceneryDirector::iterate_director();
		if (conductor->isconducting() &&
				!conductor->music_clip_is_playing())
			look_back();
	}
};

class TownApp : public Theatrix {
  	Town1 *town1;
 	Town2 *town2;
	Town3 *town3;
	Town4 *town4;
	Town5 *town5;
	Town6 *town6;
	Town7 *town7;
	Town8 *town8;
	Town9 *town9;
	Town10 *town10;
	Town11 *town11;
	Town12 *town12;
public:
	TownApp();
	~TownApp();
};

TownApp::TownApp() : Theatrix("Town")
{
	town1 = new Town1;
	town2 = new Town2;
	town3 = new Town3;
	town4 = new Town4;
	town5 = new Town5;
	town6 = new Town6;
	town7 = new Town7;
	town8 = new Town8;
	town9 = new Town9;
	town10 = new Town10;
	town11 = new Town11;
	town12 = new Town12;
}

TownApp::~TownApp()
{
	delete town12;
	delete town11;
	delete town10;
	delete town9;
	delete town8;
	delete town7;
	delete town6;
	delete town5;
	delete town4;
	delete town3;
	delete town2;
	delete town1;
}

int main()
{
	TownApp app;
	app.go();

	return 0;
}
